@@ -7,6 +7,7 @@
 #include "subread.h"
 #include "input-files.h"
 #include "core.h"
+#include "HelperFunctions.h"
 
 static struct option long_options[] =
 {
@@ -84,7 +85,8 @@ void print_usage_core_aligner()
 	SUBREADputs("  -r <string>       Name of an input read file. If paired-end, this should be");
 	SUBREADputs("                    the first read file (typically containing \"R1\"in the file");
 	SUBREADputs("                    name) and the second should be provided via \"-R\".");
-	SUBREADputs("                    Acceptable formats include gzipped FASTQ, FASTQ and FASTA.");
+	SUBREADputs("                    Acceptable formats include gzipped FASTQ, FASTQ, gzipped");
+	SUBREADputs("                    FASTA and FASTA.");
 	SUBREADputs("                    These formats are identified automatically.");
 	SUBREADputs("    ");
 	SUBREADputs("  -t <int>          Type of input sequencing data. Its values include");
@@ -663,6 +665,7 @@ int main_align(int argc , char ** argv)
 
 //	printf("SIZE_OF_ALN=%d\n", sizeof(mapping_result_t));
 //	printf("SIZE_OF_VOT=%d\n", sizeof(voting_context_t));
-	return core_main(argc, argv, parse_opts_aligner);
+	int ret = core_main(argc, argv, parse_opts_aligner);
+	return ret;
 }
 
