@@ -7,7 +7,7 @@
 #include "subread.h"
 #include "input-files.h"
 #include "core.h"
-
+#include "HelperFunctions.h"
 
 static struct option long_options[] =
 {
@@ -88,7 +88,8 @@ void print_usage_core_subjunc()
 	SUBREADputs("  -r <string>       Name of an input read file. If paired-end, this should be");
 	SUBREADputs("                    the first read file (typically containing \"R1\"in the file");
 	SUBREADputs("                    name) and the second should be provided via \"-R\".");
-	SUBREADputs("                    Acceptable formats include gzipped FASTQ, FASTQ and FASTA.");
+	SUBREADputs("                    Acceptable formats include gzipped FASTQ, FASTQ, gzipped");
+	SUBREADputs("                    FASTA and FASTA.");
 	SUBREADputs("                    These formats are identified automatically.");
 	SUBREADputs("");
 	SUBREADputs("## Optional arguments:");
@@ -694,6 +695,7 @@ int subread_subjunc_main(int argc , char ** argv)
 int main_junction(int argc , char ** argv)
 {
 #endif
-	return core_main(argc, argv, parse_opts_subjunc);
+	int ret = core_main(argc, argv, parse_opts_subjunc);
+    return ret;
 }
 
