@@ -92,6 +92,7 @@ int print_usage_gen_reads(char * pgname) {
 
 static struct option long_options[] =
 {
+	{"quiet", no_argument, 0, 'Q'},
 	{"truthInReadNames", no_argument, 0, 'T'},
 	{"simpleTranscriptId", no_argument, 0, 'C'},
 	{"summarizeFasta",  no_argument, 0, 'M'},
@@ -129,6 +130,7 @@ typedef struct {
 	int insertion_length_max;
 	int insertion_length_min;
 	float insertion_length_sigma;
+	int quiet;
 	int read_length;
 	int no_actual_reads;
 
@@ -500,6 +502,13 @@ int grc_summary_fasta(genRand_context_t * grc){
 		char clinebuf[TRANSCRIPT_FASTA_LINE_WIDTH];
 		int rlength = autozip_gets(&auto_FP, clinebuf, TRANSCRIPT_FASTA_LINE_WIDTH -1);
 		if(rlength < 1)break;
+		if(rlength > 1 && clinebuf[rlength-2] =='\r' ){
+			rlength--;
+		 	clinebuf[rlength-1]='\n';
+		 	clinebuf[rlength]=0;
+		}else if(clinebuf[rlength-1]=='\r'){
+			clinebuf[rlength-1]='\n';
+		}
 		if(rlength >= TRANSCRIPT_FASTA_LINE_WIDTH -1 || clinebuf[rlength]!='\0' || clinebuf[rlength-1]!='\n'){
 			SUBREADprintf("Error: The line width of the fasta file excessed %d bytes!\n", TRANSCRIPT_FASTA_LINE_WIDTH);
 			ret = 1;
@@ -595,7 +604,7 @@ int grc_summary_fasta(genRand_context_t * grc){
 	}
 	HashTableDestroy(reprs_tab);
 
-	if(total_rep_seq)SUBREADprintf("Warning: %d duplicate sequences were found in the input. Please check the summary table.\n",total_rep_seq);
+	if(total_rep_seq && grc->quiet==0)SUBREADprintf("Warning: %d duplicate sequences were found in the input. Please check the summary table.\n",total_rep_seq);
 
 	ArrayListDestroy(seq_name_list);
 	HashTableDestroy(seq_length_tab);
@@ -882,8 +891,11 @@ int gen_rnaseq_reads_main(int argc, char ** argv)
 
 	long long seed = -1;
 
-	while ((c = getopt_long (argc, argv, "O:TCxS:V:N:X:F:L:q:r:t:e:o:pM?", long_options, &option_index)) != -1) {
+	while ((c = getopt_long (argc, argv, "QO:TCxS:V:N:X:F:L:q:r:t:e:o:pM?", long_options, &option_index)) != -1) {
 		switch(c){
+			case 'Q':
+				grc.quiet=1;
+				break;
 			case 'x':
 				grc.no_actual_reads = 1;
 				break;
