@@ -39,9 +39,9 @@
 
 
 typedef struct {
-	char input_file_name [300];
-	char output_file_name [300];
-	char temp_file_prefix [300];
+	char input_file_name [MAX_FILE_NAME_LENGTH];
+	char output_file_name [MAX_FILE_NAME_LENGTH];
+	char temp_file_prefix [MAX_FILE_NAME_LENGTH];
 	int is_BAM_input;
 
 	unsigned long long all_reads;
@@ -68,7 +68,7 @@ void PROPMAPPED_SIGINT_hook(int param)
 	int xk1, last_slash = -1;
 	if(_PROPMAPPED_delete_tmp_prefix != NULL)
 	{
-		char del2[300], del_suffix[200], del_name[400];
+		char del2[MAX_FILE_NAME_LENGTH], del_suffix[MAX_FILE_NAME_LENGTH], del_name[MAX_FILE_NAME_LENGTH];
 		SUBREADprintf("\n\nReceived a terminal signal. The temporary files were removed.\n");
 		for(xk1=0; _PROPMAPPED_delete_tmp_prefix[xk1]; xk1++)
 		{
@@ -239,7 +239,7 @@ FILE * get_FP_by_read_name(propMapped_context * context, char * read_name)
 	FILE * ret = HashTableGet(context -> split_fp_table, NULL+1+(int)(hash_key));
 	if(!ret)
 	{
-		char fname [340];
+		char fname [MAX_FILE_NAME_LENGTH+40];
 		sprintf(fname, "%s-%llu.bin", context->temp_file_prefix, hash_key);
 		ret = f_subr_open(fname, "wb");
 		HashTablePut(context -> split_fp_table, NULL+1+(int)(hash_key), ret);
@@ -354,7 +354,7 @@ int prop_PE(propMapped_context * context)
 	int bini;
 	for(bini = 0; bini < context -> sort_buckets; bini++)
 	{
-		char fname[360];
+		char fname[MAX_FILE_NAME_LENGTH+25];
 		HashTable * rname_table = HashTableCreate(100000);
 		HashTableSetKeyComparisonFunction(rname_table , fc_strcmp_chro);
 		HashTableSetHashFunction(rname_table, fc_chro_hash);
