@@ -1089,7 +1089,7 @@ int SamBam_writer_create(SamBam_Writer * writer, char * BAM_fname, int threads,
 	{
 		writer -> bam_fp = f_subr_open(BAM_fname, "wb");
 		if(keep_in_memory){
-			char tname[350];
+			char tname[MAX_FILE_NAME_LENGTH];
 			sprintf(tname, "%s.bai", BAM_fname);
 			writer -> BAI_fp = f_subr_open(tname, "wb");
 		}
@@ -1784,7 +1784,10 @@ int SamBam_writer_sort_buff_one_compare(void * Lbin, void * Rbin){
 // return total reads in bin.
 int SamBam_writer_sort_buff_one_write(SamBam_Writer * writer, char * bin, int binlen, int thread_id){
 	int bin_cursor = 0;
-	//SUBREADprintf("WONE : BINLEN=%d, TH=%d\n", binlen, thread_id);
+	if(binlen<1){
+		SUBREADprintf("WONE : BINLEN=%d, TH=%d\n", binlen, thread_id);
+		assert(binlen >0);
+	}
 
 	ArrayList* sort_linear_pos = ArrayListCreate(1000000);
 	ArrayListSetDeallocationFunction(sort_linear_pos,  free);
@@ -1820,7 +1823,7 @@ int SamBam_writer_sort_buff_one_write(SamBam_Writer * writer, char * bin, int bi
 	memcpy(bin, nbin, binlen);
 	ArrayListDestroy(sort_linear_pos);
 
-	char tmpfname[350];
+	char tmpfname[MAX_FILE_NAME_LENGTH+40];
 	if(writer -> threads>1) subread_lock_occupy(&writer -> thread_bam_lock);
 	sprintf(tmpfname, "%s-%06d.sortedbin", writer -> tmpf_prefix, writer -> sorted_batch_id++);
 	if(writer -> threads>1) subread_lock_release(&writer -> thread_bam_lock);
@@ -1831,7 +1834,7 @@ int SamBam_writer_sort_buff_one_write(SamBam_Writer * writer, char * bin, int bi
 	}
 	free(nbin);
 	if(wlen < 1) {
-		SUBREADprintf("ERROR: no space in the temp directory. The program cannot run properly.\n");
+		SUBREADprintf("ERROR: no space (%d bytes) in the temp directory (%s).\nThe program cannot run properly.\n", binlen, tmpfname);
 		writer ->is_internal_error = 1;
 		return -1;
 	}
@@ -1968,7 +1971,7 @@ void SamBam_writer_one_thread_merge_sortedbins(SamBam_Writer * writer){
 	if(writer -> sorted_batch_id > SAMBAM_MERGE_MAX_FPS){
 		int merge_i;
 		for(merge_i = 0; merge_i < writer -> sorted_batch_id; merge_i+= SAMBAM_MERGE_MAX_FPS){
-			char tfp[360];
+			char tfp[MAX_FILE_NAME_LENGTH+50];
 			int this_size = min(SAMBAM_MERGE_MAX_FPS, writer -> sorted_batch_id - merge_i);
 			if(this_size < 2) break;
 
@@ -1982,11 +1985,11 @@ void SamBam_writer_one_thread_merge_sortedbins(SamBam_Writer * writer){
 			int current_min_batch = -1, bii;
 
 			for(bii = 0; bii < this_size; bii++){
-				char tfp[360];
+				char tfpx[MAX_FILE_NAME_LENGTH+50];
 				current_min_fps[bii] = SUBREAD_MAX_ULONGLONG;
 
-				sprintf(tfp , "%s-%06d.sortedbin", writer -> tmpf_prefix, bii + merge_i);
-				sb_fps[bii] = fopen(tfp,"r");
+				sprintf(tfpx , "%s-%06d.sortedbin", writer -> tmpf_prefix, bii + merge_i);
+				sb_fps[bii] = fopen(tfpx,"r");
 				current_min_fps[bii] = SamBam_writer_sort_bins_to_BAM_FP_pos(sb_fps[bii]);
 				if(current_min_fps[bii] < SUBREAD_MAX_ULONGLONG && current_min_fps[bii] < current_min){
 					current_min = current_min_fps[bii];
@@ -2183,7 +2186,7 @@ void SamBam_writer_sort_bins_to_BAM(SamBam_Writer * writer){
 	writer -> chunk_buffer_used = 0;
 
 	for(bii = 0; bii < writer -> sorted_batch_id; bii++){
-		char tfp[360];
+		char tfp[MAX_FILE_NAME_LENGTH+40];
 		current_min_fps[bii] = SUBREAD_MAX_ULONGLONG;
 
 		sprintf(tfp , "%s-%06d.sortedbin", writer -> tmpf_prefix, bii);
@@ -2279,7 +2282,7 @@ void SamBam_writer_sort_bins_to_BAM(SamBam_Writer * writer){
 	for(bii = 0; bii < writer -> sorted_batch_id; bii++){
 		if(!sb_fps[bii]) continue;
 
-		char tfp[360];
+		char tfp[MAX_FILE_NAME_LENGTH+40];
 		sprintf(tfp , "%s-%06d.sortedbin", writer -> tmpf_prefix, bii);
 		fclose(sb_fps[bii]);
 		unlink(tfp);
@@ -2305,8 +2308,10 @@ void SamBam_writer_finalise_thread(SamBam_Writer * writer, int thread_id){
 void SamBam_writer_finalise_one_thread(SamBam_Writer * writer){
 	if(writer -> threads < 2){
 		if(writer -> keep_in_memory){
-			SamBam_writer_sort_buff_one_write(writer, writer -> chunk_buffer, writer -> chunk_buffer_used, -1);
-			writer -> chunk_buffer_used = 0;
+			if(writer -> chunk_buffer_used>0){
+				SamBam_writer_sort_buff_one_write(writer, writer -> chunk_buffer, writer -> chunk_buffer_used, -1);
+				writer -> chunk_buffer_used = 0;
+			}
 		}else{
 			if(writer -> chunk_buffer_used)
 				SamBam_writer_add_chunk(writer, -1);
