@@ -34,34 +34,26 @@
 
 #define _gehash_hash(k) ((unsigned int)(k))
 
-int gehash_create(gehash_t * the_table, size_t expected_size, char is_small_table)
-{
+
+int gehash_create(gehash_t * the_table, size_t expected_size, char is_small_table) {
 	return gehash_create_ex(the_table, expected_size, is_small_table, SUBINDEX_VER0, 3, 0);
 }
-int gehash_create_ex(gehash_t * the_table, size_t expected_size, char is_small_table, int version_number, int index_gap, int padding)
-{
-	int expected_bucket_number;
-	int i;
 
-	memset(the_table, 0, sizeof(gehash_t));
+unsigned int calculate_buckets_by_size( size_t expected_size, int version_number, int is_small_table, int index_gap ){
 	if(expected_size ==0)
 		expected_size = GEHASH_DEFAULT_SIZE;
+	int expected_bucket_number = expected_size / GEHASH_BUCKET_LENGTH; 
 
-	// calculate the number of buckets for creating the data structure
-	expected_bucket_number = expected_size / GEHASH_BUCKET_LENGTH; 
+	if(index_gap >= 3) expected_bucket_number /=3;
 
-	if(SUBINDEX_VER1 > version_number)
-	{
+	if(SUBINDEX_VER1 > version_number) {
 		if(expected_bucket_number < 10111 && !is_small_table)expected_bucket_number = 10111;
 		else if(is_small_table)	expected_bucket_number = 4;
-	}
-	else
-	{
-		if(expected_bucket_number < 140003 )expected_bucket_number = 140003;
-	}
+	} else 
+		if(expected_bucket_number <= 0x3ffff )expected_bucket_number = 0x3ffff+4;
 
-	for (;;expected_bucket_number++)
-	{
+
+	for (;;expected_bucket_number++) {
 		int j, valid_v;
 		
 		valid_v = 1;
@@ -77,12 +69,29 @@ int gehash_create_ex(gehash_t * the_table, size_t expected_size, char is_small_t
 			break;
 	}
 
+	//SUBREADprintf("EXP_BUCKS=%u\n",expected_bucket_number);
+
+	return expected_bucket_number;
+}
+
+int gehash_create_ex(gehash_t * the_table, size_t expected_size, char is_small_table, int version_number, int index_gap, int padding) {
+	int expected_bucket_number;
+	int i;
+
+	memset(the_table, 0, sizeof(gehash_t));
+	if(expected_size ==0)
+		expected_size = GEHASH_DEFAULT_SIZE;
+
+	expected_bucket_number = calculate_buckets_by_size(expected_size, version_number, is_small_table, index_gap);
+
+	// calculate the number of buckets for creating the data structure
+
+
 	the_table -> version_number = version_number;
 	the_table -> current_items = 0;
 	the_table -> is_small_table = is_small_table;
 	the_table -> buckets_number = expected_bucket_number;
-	the_table -> buckets = (struct gehash_bucket *)
-			malloc(
+	the_table -> buckets = (struct gehash_bucket *) malloc(
 			  expected_bucket_number *
   			  sizeof(struct gehash_bucket )
 			);
@@ -374,20 +383,30 @@ int _gehash_resize_bucket_old(gehash_t * the_table , int bucket_no, char is_smal
 }
 
 #define _gehash_get_bucket(tab, key)  ( (tab) -> buckets + _gehash_hash( key ) % (tab) -> buckets_number )
+#define _gehash_get_bucketNO(tab, key)  ( _gehash_hash( key ) % (tab) -> buckets_number )
 
-int gehash_insert(gehash_t * the_table, gehash_key_t key, gehash_data_t data)
+void gehash_try_insert_measure(unsigned int * bucket_sizes, int bucket_no, gehash_key_t key){
+	unsigned int buck_this = _gehash_hash(key)%bucket_no;
+	//if(buck_this == 16305862)SUBREADprintf("Insert bucket : %u/%u -> %u at PTR=%p\n", buck_this, bucket_no, bucket_sizes[buck_this] , bucket_sizes);
+	bucket_sizes[buck_this]++;
+}
+
+int gehash_insert(gehash_t * the_table, gehash_key_t key, gehash_data_t data, unsigned int * bucket_sizes)
 {
 	struct gehash_bucket * current_bucket;
 	int is_fault = 0;
 
+	//SUBREADprintf("Insert subread : %u/%u\n",   _gehash_hash(key)% the_table->buckets_number,  the_table->buckets_number);
 	current_bucket = _gehash_get_bucket (the_table, key);
 
 	if(the_table->version_number == SUBINDEX_VER0)
 	{
-		if (current_bucket->current_items >= current_bucket->space_size)
-		{
-
+		if (current_bucket->current_items >= current_bucket->space_size) {
 			int bucket_number;
+			if(bucket_sizes){
+				SUBREADprintf("Bucket size was wrongly calculated!\n");
+				return 1;
+			}
 
 			bucket_number = _gehash_hash(key) % the_table -> buckets_number;
 			is_fault = _gehash_resize_bucket(the_table, bucket_number, the_table->is_small_table);
@@ -399,9 +418,21 @@ int gehash_insert(gehash_t * the_table, gehash_key_t key, gehash_data_t data)
 	else
 	{
 		short step_number = key/the_table -> buckets_number;
+		if(current_bucket-> new_item_keys == NULL && bucket_sizes){
+			unsigned int bkno = _gehash_get_bucketNO(the_table, key) ;
+			unsigned char * mem_block = malloc((sizeof(gehash_data_t) + sizeof(short)) * bucket_sizes[ bkno ] ); 
+			current_bucket->item_values = ( gehash_data_t * )mem_block;
+			current_bucket->new_item_keys = ( short*)(mem_block + sizeof(gehash_data_t) *  bucket_sizes[ bkno ]) ;
+			current_bucket->current_items = 0;
+			current_bucket->space_size = bucket_sizes[ bkno ];
+			the_table -> free_item_only = 1;
+		}
 
-		if (current_bucket->current_items >= current_bucket->space_size)
-		{
+		if (current_bucket->current_items >= current_bucket->space_size) {
+			if(bucket_sizes){
+				SUBREADprintf("Bucket [%d] size was wrongly calculated : %d >= %u!\n",  _gehash_get_bucketNO (the_table, key), current_bucket->current_items,  current_bucket->space_size);
+				return 1;
+			}
 
 			int bucket_number;
 			bucket_number = _gehash_hash(key) % the_table -> buckets_number;
@@ -411,8 +442,8 @@ int gehash_insert(gehash_t * the_table, gehash_key_t key, gehash_data_t data)
 				return 1;
 		}
 		current_bucket->new_item_keys[current_bucket->current_items] = step_number;
-
 	}
+
 	current_bucket->item_values[current_bucket->current_items] = data;
 	current_bucket->current_items ++;
 	the_table ->current_items ++;
@@ -451,7 +482,7 @@ int gehash_insert_limited(gehash_t * the_table, gehash_key_t key, gehash_data_t
 
 	}
 
-	gehash_insert(the_table, key, data);
+	gehash_insert(the_table, key, data, NULL);
 	return 0;
 }
 	
@@ -588,8 +619,11 @@ size_t gehash_go_q_CtoT(gehash_t * the_table, gehash_key_t key, int offset, int
 							mutation_stack[j] = 0;
 						is_end2 = 0;
 
-						if(key != mutation_key)
-							ret+=gehash_go_q(the_table, mutation_key, offset, read_len, is_reversed, vote, indel_tolerance, subread_number, low_border, high_border);
+						if(key != mutation_key){
+							int retone;
+							retone = gehash_go_q(the_table, mutation_key, offset, read_len, is_reversed, vote, indel_tolerance, subread_number, low_border, high_border);
+							ret+=retone;
+						}
 						mutation_stack[i] ++;
 						break;
 					}
@@ -1046,6 +1080,201 @@ size_t gehash_go_q(gehash_t * the_table, gehash_key_t raw_key, int offset, int r
 	}
 }
 
+
+
+size_t gehash_go_X(gehash_t * the_table, gehash_key_t raw_key, int offset, int read_len, int is_reversed, gene_vote_t * vote, int indel_tolerance, int subread_number, unsigned int low_border, unsigned int high_border, int run_round, unsigned int * shift_indel_locs, unsigned int * shift_indel_NO){
+	if(0)if(the_table->version_number == SUBINDEX_VER0){
+		SUBREADprintf("ERROR: the version of the index is too old.\n");
+		assert(the_table->version_number != SUBINDEX_VER0);
+		return -1;
+	} 
+
+	// VER_1
+	// VER_2
+
+	struct gehash_bucket * current_bucket;
+	int i = 0, items;
+
+	short *current_keys;//, *endp12;
+	short key = raw_key / the_table->buckets_number;
+
+	current_bucket = _gehash_get_bucket (the_table, raw_key);
+	items = current_bucket -> current_items;
+	current_keys = current_bucket -> new_item_keys;
+	
+	if(!items) return 0;
+
+//#warning "======== MAKE SURE THAT '-1' IS CORRECT ============"
+	int imin=0, imax=items - 1;
+	int last_accepted_index;
+
+	while(1)
+	{
+		last_accepted_index=(imin+imax)/2;
+		short current_key = current_keys[last_accepted_index];
+		if(current_key>key)
+		{
+			imax = last_accepted_index - 1;
+		}
+		else if(current_key<key)
+		{
+			imin = last_accepted_index + 1;
+		}
+		else
+			break;
+
+		if(imax<imin)
+			return 0;
+		
+	}
+
+
+	int subread_number_P1 =  subread_number + 1;
+	int of_p_16 = offset + 16;
+	is_reversed = is_reversed?IS_NEGATIVE_STRAND:0;
+	int start_scan_idx = last_accepted_index, scan_step = 0;
+
+	{
+		int ii_end = INDEL_SEGMENT_SIZE;
+		if(indel_tolerance>5) ii_end=(indel_tolerance % INDEL_SEGMENT_SIZE)?(indel_tolerance - indel_tolerance%INDEL_SEGMENT_SIZE+INDEL_SEGMENT_SIZE):indel_tolerance;
+
+		//for (; last_accepted_index<items && current_keys[last_accepted_index] == key ; last_accepted_index++){
+		while(1){
+			unsigned int kv = current_bucket->item_values[last_accepted_index] - offset;
+			int iix, offsetX2, offsetX, datalen, datalen2;
+			offsetX2 = offsetX = _index_vote_tol(kv);
+			datalen = datalen2 = vote -> items[offsetX2];
+			unsigned int * dat2, *dat;
+			dat = dat2 = vote -> pos[offsetX2];
+			int found_some = 0;
+
+			//SUBREADprintf("You can find KV at %u\n", kv);
+
+			for(iix = 0; iix<=ii_end; iix = iix>0?-iix:(-iix+INDEL_SEGMENT_SIZE)) {
+
+				if(iix) {
+					offsetX = _index_vote_tol(kv+iix);
+					datalen = vote -> items[offsetX];
+					dat = vote -> pos[offsetX];
+				}
+
+
+				if(!datalen)continue;
+
+				for (i=0;i<datalen;i++)
+				{
+					int dist0 = kv-dat[i];
+					int applied_indel_tol = ( run_round>0 && vote -> marked_shift_indel[offsetX][i])? 0: indel_tolerance ;
+					if( dist0 >= -applied_indel_tol && dist0 <= applied_indel_tol && is_reversed == vote->masks[offsetX][i]){
+						int toli =  vote -> toli[offsetX][i];
+
+						if(run_round == 0 && toli >0 && dist0 ==0 && ! vote -> marked_shift_indel[offsetX][i]){
+							vote -> marked_shift_indel[offsetX][i] = 1;
+							shift_indel_locs[(* shift_indel_NO)++] = dat[i];
+						}
+
+						if( subread_number_P1 == vote -> last_subread_cluster[offsetX][i]  && toli >0){
+							int move_dist = 0;
+							if( toli >=3 ) move_dist = vote -> indel_recorder[offsetX][i][toli-3+2];
+							int new_dist = move_dist;
+							move_dist -= vote -> indel_recorder[offsetX][i][toli+2];
+							new_dist -= dist0;
+							if(abs(move_dist) > abs(new_dist)){
+								toli -= 3;
+								vote -> toli[offsetX][i] = toli;
+								vote -> last_subread_cluster[offsetX][i]--;
+								vote -> votes[offsetX][i] --;
+							}
+						}
+	
+						if(subread_number_P1 <= vote -> last_subread_cluster[offsetX][i]) continue;
+						gene_vote_number_t test_max = (vote->votes[offsetX][i]);
+						test_max += 1;
+						vote -> votes[offsetX][i] = test_max;
+
+						if (offset +16 > vote->coverage_end [offsetX][i])
+							vote->coverage_end [offsetX][i] = of_p_16;
+	
+	
+						if (dist0 ==  vote->current_indel_cursor[offsetX][i]){
+							vote -> indel_recorder[offsetX][i][toli+1] = subread_number_P1;
+						} else {
+							toli +=3;
+							if (toli < MAX_INDEL_SECTIONS*3)
+							{
+								vote -> toli[offsetX][i] = toli;
+								vote -> indel_recorder[offsetX][i][toli] = subread_number_P1; 
+								vote -> indel_recorder[offsetX][i][toli+1] = subread_number_P1;
+								vote -> indel_recorder[offsetX][i][toli+2] = dist0; 
+									
+								if(toli < MAX_INDEL_SECTIONS*3-3) vote -> indel_recorder[offsetX][i][toli+3]=0;
+							}
+							vote->current_indel_cursor [offsetX][i] = (char)dist0;
+						}
+
+						vote -> last_subread_cluster[offsetX][i] = subread_number_P1;
+						if(vote->max_vote < test_max)vote->max_vote = test_max;
+						found_some = 1;
+						break;
+					}
+				}
+				if(found_some) break;
+			}
+
+			if (!found_some) {
+				if (kv >=low_border && kv <= high_border && datalen2<GENE_VOTE_SPACE) {
+					vote -> items[offsetX2] ++;
+					dat2[datalen2] = kv;
+					vote -> masks[offsetX2][datalen2] = is_reversed;
+					vote -> votes[offsetX2][datalen2] = 1;
+					vote -> toli[offsetX2][datalen2] = 0;
+					vote -> marked_shift_indel[offsetX2][datalen2] = 0;
+					if(run_round>0){
+						int kk;
+						for(kk = 0; kk < * shift_indel_NO ; kk++){
+							if( kv >= shift_indel_locs[kk] - indel_tolerance && kv <= shift_indel_locs[kk] + indel_tolerance ){
+								vote -> marked_shift_indel[offsetX2][datalen2] = 1;
+								break;
+							}
+						}
+					}
+	
+					// data structure of recorder:
+					// {unsigned char subread_start; unsigned char subread_end, char indel_offset_from_start}
+					// All subread numbers are added with 1 for not being 0.
+
+					vote -> indel_recorder[offsetX2][datalen2][0] = vote -> indel_recorder[offsetX2][datalen2][1] = subread_number_P1;
+					vote -> indel_recorder[offsetX2][datalen2][2] = 0;
+					vote -> indel_recorder[offsetX2][datalen2][3] = 0;
+					vote->current_indel_cursor [offsetX2][datalen2] = 0;
+					vote->coverage_start [offsetX2][datalen2] = offset;
+					vote->coverage_end [offsetX2][datalen2] = of_p_16;
+					vote -> last_subread_cluster[offsetX2][datalen2] = subread_number_P1;
+	
+					if (vote->max_vote==0)
+						vote->max_vote = 1;
+				}
+			}
+
+			if(! scan_step){
+				last_accepted_index++;
+				if(last_accepted_index == items || current_keys[last_accepted_index]!= key){
+					scan_step = 1;
+					last_accepted_index = start_scan_idx;
+				}
+			}
+			if(scan_step){
+				last_accepted_index --;
+				if(last_accepted_index <0 || current_keys[last_accepted_index]!= key) break;
+			}
+		}
+	}	
+	return 1;
+}
+
+
+
+
 void select_best_vote(gene_vote_t * vote)
 {
 	int i,j;
@@ -1789,11 +2018,10 @@ void gehash_destory(gehash_t * the_table)
 		struct gehash_bucket * current_bucket = &(the_table -> buckets[i]);
 		if (current_bucket -> space_size > 0)
 		{
-			free (current_bucket -> item_keys);
+			if(0==the_table->free_item_only)free (current_bucket -> item_keys);
 			free (current_bucket -> item_values);
 		}
 	}
-
 	free (the_table -> buckets);
 
 	the_table -> current_items = 0;
