@@ -64,7 +64,7 @@
 #define MAX_READ_LENGTH ( 1210 )
 #define MAX_READ_NAME_LEN 200 
 #define MAX_CHROMOSOME_NAME_LEN 200 
-#define MAX_FILE_NAME_LENGTH 300
+#define MAX_FILE_NAME_LENGTH (1000)
 #define FEATURE_NAME_LENGTH 256 
 
 //#warning "============== REMOVE '*1.2' FROM THE NEXT LINE ================"
@@ -272,6 +272,7 @@ typedef struct {
 	int index_gap;
 	int padding;
 	char * malloc_ptr;
+	int free_item_only;
 } gehash_t;
 
 
@@ -293,14 +294,15 @@ typedef struct {
 	gene_quality_score_t max_quality;
 	gene_vote_number_t max_indel_recorder[MAX_INDEL_TOLERANCE*3];
 	gene_vote_number_t * max_tmp_indel_recorder;
-	short max_mask;
+	int max_mask;
 	gene_vote_number_t noninformative_subreads;
 
-        unsigned short items[GENE_VOTE_TABLE_SIZE];
-        unsigned int pos [GENE_VOTE_TABLE_SIZE][GENE_VOTE_SPACE];
-        gene_vote_number_t votes [GENE_VOTE_TABLE_SIZE][GENE_VOTE_SPACE];
-        gene_quality_score_t quality [GENE_VOTE_TABLE_SIZE][GENE_VOTE_SPACE];
-	short masks [GENE_VOTE_TABLE_SIZE][GENE_VOTE_SPACE];
+	unsigned short items[GENE_VOTE_TABLE_SIZE];
+	unsigned int pos [GENE_VOTE_TABLE_SIZE][GENE_VOTE_SPACE];
+	int masks [GENE_VOTE_TABLE_SIZE][GENE_VOTE_SPACE];
+	int marked_shift_indel[GENE_VOTE_TABLE_SIZE][GENE_VOTE_SPACE];
+	gene_vote_number_t votes [GENE_VOTE_TABLE_SIZE][GENE_VOTE_SPACE];
+	gene_quality_score_t quality [GENE_VOTE_TABLE_SIZE][GENE_VOTE_SPACE];
 	gene_vote_number_t last_subread_cluster [GENE_VOTE_TABLE_SIZE][GENE_VOTE_SPACE];
 	gene_vote_number_t indel_recorder [GENE_VOTE_TABLE_SIZE][GENE_VOTE_SPACE][MAX_INDEL_TOLERANCE*3];
 	char current_indel_cursor[GENE_VOTE_TABLE_SIZE][GENE_VOTE_SPACE];
@@ -425,7 +427,19 @@ typedef struct{
 	unsigned int in_block_text_offset;
 } seekable_position_t;
 
+typedef struct {
+	unsigned long long read_number;
+	int lane_id;
 
+	union{
+		seekable_position_t ** pos_of_bclgzs;
+		unsigned long long * pos_of_bcls;
+	};
+	union{
+		seekable_position_t * pos_of_filtergz;
+		unsigned long long pos_of_filter;
+	};
+} input_BLC_pos_t;
 
 typedef struct {
 	char filename [300];
@@ -440,6 +454,7 @@ typedef struct{
 	union{
 		unsigned long long simple_file_position;
 		seekable_position_t seekable_gzip_position;
+		input_BLC_pos_t BCL_position;
 	};
 	char gzfa_last_name[MAX_READ_NAME_LEN];
 } gene_inputfile_position_t;
